import 'package:aliyun_oss_platform_interface/aliyun_oss_platform_interface.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:freezed_annotation/freezed_annotation.dart';

export 'oss/export_oss.dart';

const MethodChannel _methodChannel =
    MethodChannel('com.wombat/aliyun_oss_plugin');

class MethodChannelAliyunOSS extends AliyunOSSPlatform {
  MethodChannelAliyunOSS() {
    _methodChannel.setMethodCallHandler(_methodCallHandler);
  }

  final List<UploadMethodCallListener> _uploadMethodCallListeners = [];
  final List<DownloadMethodCallListener> _downloadMethodCallListeners = [];

  void _notifyUploadListeners(
      void Function(UploadMethodCallListener listener) callback) {
    _notifyListeners(callback, _uploadMethodCallListeners);
  }

  void _notifyDownloadListeners(
      void Function(DownloadMethodCallListener listener) callback) {
    _notifyListeners(callback, _downloadMethodCallListeners);
  }

  void _notifyListeners<T>(
      void Function(T listener) callback, List<T> listeners) {
    UnmodifiableListView(listeners).forEach((element) {
      if (listeners.contains(element)) {
        callback(element);
      }
    });
  }

  _toMap(dynamic data) {
    return (data as Map<dynamic, dynamic>)
        .map((key, value) => MapEntry('$key', value));
  }

  Future<dynamic> _methodCallHandler(MethodCall methodCall) async {
    debugPrint('回调方法: ${methodCall.method}, 参数: ${methodCall.arguments}');
    switch (methodCall.method) {

      /// 上传文件
      case 'upload_init':
        UploadInitData initData =
            UploadInitData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传初始化: ${initData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.initListener?.call(initData));
        break;
      case "upload_progress":
        UploadProgressData progressData =
            UploadProgressData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传进度: ${progressData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.progressListener?.call(progressData));
        break;
      case "upload_success":
        UploadSuccessData successData =
            UploadSuccessData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传成功: ${successData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.successListener?.call(successData));
        break;
      case "upload_fail":
        UploadFailData failData =
            UploadFailData.fromJson(_toMap(methodCall.arguments));
        debugPrint("上传失败: ${failData.toJson()}");
        _notifyUploadListeners(
            (listener) => listener.failListener?.call(failData));
        break;
      case "upload_pause":
        _notifyUploadListeners((listener) =>
            listener.pauseListener?.call(methodCall.arguments["objectKey"]));
        break;
      case "upload_resume":
        _notifyUploadListeners((listener) =>
            listener.resumeListener?.call(methodCall.arguments["objectKey"]));
        break;

      /// 下载文件
      case 'download_init':
        DownloadInitData initData =
            DownloadInitData.fromJson(_toMap(methodCall.arguments));
        debugPrint("下载初始化: ${initData.toJson()}");
        _notifyDownloadListeners(
            (listener) => listener.initListener?.call(initData));
        break;
      case "download_progress":
        DownloadProgressData progressData =
            DownloadProgressData.fromJson(_toMap(methodCall.arguments));
        debugPrint("下载进度: ${progressData.toJson()}");
        _notifyDownloadListeners(
            (listener) => listener.progressListener?.call(progressData));
        break;
      case "download_success":
        DownloadSuccessData successData =
            DownloadSuccessData.fromJson(_toMap(methodCall.arguments));
        debugPrint("下载成功: ${successData.toJson()}");
        _notifyDownloadListeners(
            (listener) => listener.successListener?.call(successData));
        break;
      case "download_fail":
        DownloadFailData failData =
            DownloadFailData.fromJson(_toMap(methodCall.arguments));
        debugPrint("下载失败: ${failData.toJson()}");
        _notifyDownloadListeners(
            (listener) => listener.failListener?.call(failData));
        break;
      case "download_pause":
        _notifyDownloadListeners((listener) =>
            listener.pauseListener?.call(methodCall.arguments["objectKey"]));
        break;
      case "download_resume":
        _notifyDownloadListeners((listener) =>
            listener.resumeListener?.call(methodCall.arguments["objectKey"]));
        break;
    }
  }

  @override
  Future<String?> saveFileToDownload(String fileName, String mimeType) async {
    return (await _methodChannel.invokeMethod('saveFileToDownload', {
      'fileName': fileName,
      'mimeType': mimeType,
    }));
  }

  @override
  Future<String> getMd5Str(String filePath) async {
    return (await _methodChannel
        .invokeMethod('getMd5Str', {'filePath': filePath})) as String;
  }

  @override
  init(String endpoint, OSSCredentialProvider provider) {
    return _methodChannel.invokeMethod(
        'init', provider.toJson()..addAll({"endpoint": endpoint}));
  }

  @override
  Future<dynamic> upload(String bucketName) async {
    return _methodChannel.invokeMethod('upload', {"bucketName": bucketName});
  }

  @override
  uploadWithPath(String bucketName, String filePath, String md5, String uuid) {
    return _methodChannel.invokeMethod('uploadWithPath', {
      "bucketName": bucketName,
      "filePath": filePath,
      "md5": md5,
      "uuid": uuid,
    });
  }

  @override
  Future<String?> pauseUpload(String objectKey, String uuid) {
    return _methodChannel
        .invokeMethod('pause_upload', {'objectKey': objectKey, 'uuid': uuid});
  }

  @override
  Future<String?> resumeUpload(String objectKey, String uuid) {
    return _methodChannel
        .invokeMethod('resume_upload', {'objectKey': objectKey, 'uuid': uuid});
  }

  @override
  Future<String?> cancelUpload(String objectKey, String uuid) {
    return _methodChannel
        .invokeMethod('cancel_upload', {'objectKey': objectKey, 'uuid': uuid});
  }

  @override
  Future<String?> cancelAllUpload() {
    return _methodChannel.invokeMethod('cancel_all_upload');
  }

  @override
  download(String? bucketName, String md5, String fileName, String mimeType) {
    return _methodChannel.invokeMethod('download', {
      "bucketName": bucketName,
      "md5": md5,
      "fileName": fileName,
      "mimeType": mimeType,
    });
  }

  @override
  Future<String?> pauseDownload(String objectKey) {
    return _methodChannel
        .invokeMethod('pause_download', {'objectKey': objectKey});
  }

  @override
  Future<String?> resumeDownload(String objectKey) {
    return _methodChannel
        .invokeMethod('resume_Download', {'objectKey': objectKey});
  }

  @override
  void addUploadMethodCallListener(UploadMethodCallListener listener) {
    _uploadMethodCallListeners.add(listener);
  }

  @override
  void removeUploadMethodCallListener(UploadMethodCallListener listener) {
    _uploadMethodCallListeners.remove(listener);
  }

  @override
  void addDownloadMethodCallListener(DownloadMethodCallListener listener) {
    _downloadMethodCallListeners.add(listener);
  }

  @override
  void removeDownloadMethodCallListener(DownloadMethodCallListener listener) {
    _downloadMethodCallListeners.remove(listener);
  }
}
